/* kadchart.h
   Functions for handling step charts

Copyright 2004 Damian Yerrick

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, provided that the above copyright notice(s) and this permission notice appear in all copies of the Software and that both the above copyright notice(s) and this permission notice appear in supporting documentation.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

Except as contained in this notice, the name of a copyright holder shall not be used in advertising or otherwise to promote the sale, use or other dealings in this Software without prior written authorization of the copyright holder.

*/


#ifndef KADCHART_H
#define KADCHART_H

#include <stdlib.h>
#include <math.h>

/* anything used only by the PC is a KADReal;
   anything that the GBA sees is a KADFixed */

typedef float KADReal;
#define KADREAL(x) (x)
#define KADUNREAL(x) (x)

typedef long int KADFixed;
#define KADFIX(x) ((KADFixed)floor( (x) * 65536.0 + 0.5 ))
#define KADUNFIX(x) ((x) * (1.0/65536.0))

typedef struct KADV
{
  void *data;
  size_t size;
  size_t num;
  size_t max_num;
} KADV;

typedef struct KADBPMChange
{
  unsigned int row;
  KADReal bpm;
} KADBPMChange;

typedef struct KADStop
{
  unsigned int row;
  KADReal duration;
} KADStop;

typedef struct KADParsedRow
{
  unsigned short measure;
  unsigned char row;
  unsigned char column;  /* bit 7 set for a freeze */
} KADParsedRow;

typedef struct KADChart
{
  KADV *steps;  /* of KADParsedRow */
  char steps_type;  /* 'S', 'D', 'C', '6' */
  unsigned char difficulty;  /* in feet */
  unsigned char difficulty_type;  /* 1 beginner through 5 oni or 0 unrecognized */
  char reserved;
} KADChart;

typedef struct KADSync
{
  KADV *bpms;  /* of KADBPMChange */
  KADV *stops;  /* of KADStop */
  KADFixed min_real_speed, max_real_speed;
  KADFixed min_disp_speed, max_disp_speed;
  KADFixed sample_start, sample_duration;
} KADSync;

/* In KAD, gap is always 0, and initial BPM is always 60. To override
   these, put a stop and a BPM change at row 0.  dwiparse.c handles this.
*/
typedef struct KADParsedSong
{
  char title[64];
  char artist[64];
  KADSync *sync;
  KADV *charts;  /* of KADChart * */
} KADParsedSong;

extern KADParsedSong sng;  /* FIXME: once parser works, convert it to pass this */


KADV *KADV_new(size_t size, size_t initial_max_num);
void KADV_delete(KADV *c);
int KADV_growto(KADV *c, size_t n);
int KADV_growby(KADV *c, size_t n);

int KADChart_AddRow(KADChart *c, 
                    unsigned int measure, unsigned int row, unsigned int col);
KADChart *KADChart_new(int difficulty);
void KADChart_delete(KADChart *c);
KADChart *KADChart_merge(const KADChart *a, const KADChart *b, int shift);

KADSync *KADSync_new(void);
void KADSync_delete(KADSync *c);
int KADSync_AddBPM(KADSync *c, 
                   unsigned int row, KADReal speed);
int KADSync_AddStop(KADSync *c, 
                    unsigned int row, KADReal duration);

KADParsedSong *KADParsedSong_new(void);
void KADParsedSong_delete(KADParsedSong *c);
int KADParsedSong_AddChart(KADParsedSong *song, KADChart *c);

#endif
